@@ -2,6 +2,24 @@
 #' @include AllGenerics.R
 NULL
 
+# Internal environment =========================================================
+the <- new.env(parent = emptyenv())
+the$margin <- 1       # Updated by prepare_plot()
+the$axes <- c(1, 2)   # Updated by prepare_plot()
+the$principal <- TRUE # Updated by prepare_plot()
+
+get_margin <- function(...) {
+  get("margin", envir = the)
+}
+
+get_axes <- function(...) {
+  get("axes", envir = the)
+}
+
+get_principal <- function(...) {
+  get("principal", envir = the)
+}
+
 # Rows =========================================================================
 #' @export
 #' @rdname viz_individuals
@@ -361,7 +379,7 @@ viz_points <- function(x, margin, axes, ...,
 
   ## Add ellipse
   if (is.list(ellipse) && length(ellipse) > 0) {
-    args_ell <- list(x = x, group = group, margin = margin, axes = axes,
+    args_ell <- list(x = x, group = group,
                      color = color, fill = FALSE, symbol = FALSE)
     ellipse <- modifyList(args_ell, val = ellipse)
     do.call(viz_ellipses, ellipse)
@@ -369,7 +387,7 @@ viz_points <- function(x, margin, axes, ...,
 
   ## Add convex hull
   if (isTRUE(hull)) {
-    args_hull <- list(x = x, group = group, margin = margin, axes = axes,
+    args_hull <- list(x = x, group = group,
                       color = color, fill = FALSE, symbol = FALSE)
     do.call(viz_hull, args_hull)
   }
@@ -498,11 +516,10 @@ prepare_plot <- function(x, margin, ..., axes = c(1, 2), active = TRUE,
   arkhe::assert_scalar(sup, "logical")
   arkhe::assert_scalar(principal, "logical")
 
-  ## /!\ Backward compatibility /!\
-  high <- list(...)$highlight
-  if (length(high) == 1) {
-    if (high == "observation") extra_quali <- high else extra_quanti <- high
-  }
+  ## Set margin and axes
+  assign("margin", value = margin, envir = the)
+  assign("axes", value = axes, envir = the)
+  assign("principal", value = principal, envir = the)
 
   ## Prepare data
   data <- augment(x, margin = margin, axes = axes, principal = principal)
@@ -511,7 +528,8 @@ prepare_plot <- function(x, margin, ..., axes = c(1, 2), active = TRUE,
   ## Recode
   data$observation <- ifelse(data$supplementary, "suppl.", "active")
 
-  ## Recycle graphical parameters if of length one
+  ## Set graphical parameters
+  ## (recycle if of length one)
   dots <- list(...)
   col <- recycle(dots$col %||% graphics::par("col"), n)
   bg <- recycle(dots$bg %||% graphics::par("bg"), n)
@@ -536,10 +554,15 @@ prepare_plot <- function(x, margin, ..., axes = c(1, 2), active = TRUE,
     arkhe::assert_type(extra_quanti, "numeric")
     arkhe::assert_length(extra_quanti, n)
     ## Continuous scales
-    if (!isFALSE(color)) col <- khroma::palette_color_continuous(colors = color)(extra_quanti)
-    if (!isFALSE(fill)) bg <- khroma::palette_color_continuous(colors = fill)(extra_quanti)
-    if (!isFALSE(size)) cex <- khroma::palette_size_sequential(range = size)(extra_quanti)
-    if (!isFALSE(line_width)) lwd <- khroma::palette_size_sequential(range = line_width)(extra_quanti)
+    ## (ignored if col, bg, cex and lwd are set by user)
+    if (is.null(dots$col) && !isFALSE(color))
+      col <- khroma::palette_color_continuous(colors = color)(extra_quanti)
+    if (is.null(dots$bg) && !isFALSE(fill))
+      bg <- khroma::palette_color_continuous(colors = fill)(extra_quanti)
+    if (is.null(dots$cex) && !isFALSE(size))
+      cex <- khroma::palette_size_sequential(range = size)(extra_quanti)
+    if (is.null(dots$lwd) && !isFALSE(line_width))
+      lwd <- khroma::palette_size_sequential(range = line_width)(extra_quanti)
   } else {
     extra_quanti <- rep(NA_real_, n)
   }
@@ -562,10 +585,15 @@ prepare_plot <- function(x, margin, ..., axes = c(1, 2), active = TRUE,
     extra_quali <- as.vector(extra_quali)
     arkhe::assert_length(extra_quali, n)
     ## Discrete scales
-    if (!isFALSE(color)) col <- khroma::palette_color_discrete(colors = color)(extra_quali)
-    if (!isFALSE(fill)) bg <- khroma::palette_color_discrete(colors = fill)(extra_quali)
-    if (!isFALSE(symbol)) pch <- khroma::palette_shape(symbols = symbol)(extra_quali)
-    if (!isFALSE(line_type)) lty <- khroma::palette_line(types = line_type)(extra_quali)
+    ## (ignored if col, bg, pch and lty are set by user)
+    if (is.null(dots$col) && !isFALSE(color))
+      col <- khroma::palette_color_discrete(colors = color)(extra_quali)
+    if (is.null(dots$bg) && !isFALSE(fill))
+      bg <- khroma::palette_color_discrete(colors = fill)(extra_quali)
+    if (is.null(dots$pch) && !isFALSE(symbol))
+      pch <- khroma::palette_shape(symbols = symbol)(extra_quali)
+    if (is.null(dots$lty) && !isFALSE(line_type))
+      lty <- khroma::palette_line(types = line_type)(extra_quali)
   } else {
     extra_quali <- rep(NA_character_, n)
   }
