@@ -2,11 +2,7 @@ Sys.setenv(LANGUAGE = "en") # Force locale
 
 if (at_home()) {
   using("tinysnapshot")
-  options(tinysnapshot_device = "svglite")
-  options(tinysnapshot_height = 7) # inches
-  options(tinysnapshot_width = 7)
-  options(tinysnapshot_tol = 300) # pixels
-  options(tinysnapshot_os = "Linux")
+  source("helpers.R")
 
   data("iris")
 
